# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test template tags
"""
from datetime import datetime
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.template import Template, Context
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.translation import activate

from ..models import Category, Page
from .mixins import TDCMSTestMixin


class TDCMSTemplateTagTests(TDCMSTestMixin, TestCase):
    """
    Tests for the td_cms_tags template tags library
    """
    def setUp(self):
        activate('en')
        self._set_category_tree()

    def test_get_parent_categories(self):
        """
        Retrieve visible categories to display in a (mptt) rendered menu
        """
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_parent_categories as categories %}"
            "{% for category in categories %}"
            "{% if category.is_child_node %}"
            "> {{ category }} "
            "{% else %}"
            "{{ category }} "
            "{% endif %}"
            "{% endfor %}"
        )

        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = "".join(
            "Django books > Testing in Django > Jazz with Django "
            "Django tee > Keep calm > Poney ride "
            "Django stickers > Poney mascotte > Django logo "
        )
        self.assertEqual(out, expected)

        # Test with a hidden category in menu
        Category.objects.filter(name='Keep calm').update(visible_in_menu=False)
        out = Template(template).render(Context())
        expected = "".join(
            "Django books > Testing in Django > Jazz with Django "
            "Django tee > Poney ride "
            "Django stickers > Poney mascotte > Django logo "
        )
        self.assertEqual(out, expected)

    def test_get_highlighted_pages(self):
        """
        Get a queryset of all highlighted pages in templates
        """
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_highlighted_pages as pages %}"
            "{% for page in pages %}"
            "{{ page }} - "
            "{% endfor %}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = "".join(
            "Django book (vol. 8) - "
            "Django book (vol. 16) - "
            "Django book (vol. 24) - "
            "Django book (vol. 32) - "
        )
        self.assertEqual(out, expected)

    def test_get_category(self):
        """
        Get a category object by its slug in a template. We test two different
        languages (fr, en).
        """
        # Add translations for our category of interest
        category = Category.objects.get(name='Jazz with Django')
        category.name_fr = u'Jaser avec Django'
        category.save()

        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'jazz-with-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = category.name
        self.assertEqual(out, expected)

        # Switch to french
        activate('fr')

        # Test the french slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'jaser-avec-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = category.name_fr
        self.assertEqual(out, expected)

    def test_get_category_with_unknown_slug(self):
        """
        Try to get a category with a wrong slug
        """
        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_category 'swing-with-django' as category %}"
            "{{ category }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = u"None"
        self.assertEqual(out, expected)

    def test_get_page(self):
        """
        Get a page object by its slug in a template. We test two different
        languages (fr, en).
        """
        # Add translations for our category of interest
        page = Page.objects.get(title='Django book (vol. 10)')
        page.title_fr = u'Le livre Django (vol. 10)'
        page.save()

        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'django-book-vol-10' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = page.title
        self.assertEqual(out, expected)

        # Switch to french
        activate('fr')

        # Test the french slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'le-livre-django-vol-10' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = page.title_fr
        self.assertEqual(out, expected)

    def test_get_page_with_unknown_slug(self):
        """
        Try to get a page with a wrong slug
        """
        # Test the english (default) slug
        template = "".join(
            "{% load td_cms_tags %}"
            "{% get_page 'php-book-vol-1' as page %}"
            "{{ page }}"
        )
        # Test with all categories visible in menu
        out = Template(template).render(Context())
        expected = u"None"
        self.assertEqual(out, expected)

    def test_timestamp_filter(self):
        """
        Get a timestamp from a datetime.datetime object
        """
        date = datetime(year=1980, month=10, day=14, hour=18, minute=30)
        template = "".join(
            "{% load td_cms_tags %}"
            "{{ date|timestamp }}"
        )
        # Test the datetime to timestamp conversion
        out = Template(template).render(Context({'date': date}))
        expected = u"340392600"
        self.assertEqual(out, expected)

    def test_timestamp_filter_without_datetime_instance(self):
        """
        Try to get a timestamp from a non datetime.datetime object, should be
        returned identical.
        """
        date = u"14/10/1980"
        template = "".join(
            "{% load td_cms_tags %}"
            "{{ date|timestamp }}"
        )
        # Test the datetime to timestamp conversion
        out = Template(template).render(Context({'date': date}))
        expected = date
        self.assertEqual(out, expected)

    @override_settings(TD_CMS_DISQUS_SHORTNAME=None)
    def test_show_disqus_thread_while_not_configured(self):
        """
        Test the disqus thread inclusion while not configured
        """
        page = Page.objects.get(title='Django book (vol. 10)')
        page.allow_comments = True
        page.save()

        response = self.client.get(page.get_absolute_url())
        message_list = [m for m in response.context['messages']]
        message = message_list[0]
        expected_message = u"".join(
            "TD_CMS_DISQUS_SHORTNAME parameter is missing in your settings"
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(len(message_list), 1)
        self.assertEqual(expected_message, message.message)
        self.assertEqual(message.level, messages.WARNING)

        self.assertNotContains(response, '<div id="disqus_thread">')

    @override_settings(TD_CMS_DISQUS_SHORTNAME='tailordev')
    def test_show_disqus_thread_while_configured(self):
        """
        Test the disqus thread inclusion while configured
        """
        page = Page.objects.get(title='Django book (vol. 10)')
        page.allow_comments = True
        page.save()

        response = self.client.get(page.get_absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['messages']), 0)
        self.assertContains(response, '<div id="disqus_thread">')

    @override_settings(TD_CMS_DISQUS_SHORTNAME='tailordev')
    def test_show_disqus_thread_rendering(self):
        """
        Test the disqus thread inclusion while not configured
        """
        page = Page.objects.get(title='Django book (vol. 10)')
        template = "".join(
            "{% load td_cms_tags %}"
            "{% show_disqus_thread %}"
        )
        out = Template(template).render(Context({'page': page}))
        expected = "var disqus_shortname = 'tailordev';"
        self.assertIn(expected, out)
        expected = "var disqus_identifier = '%d-%s';" % (page.id, page.slug)
        self.assertIn(expected, out)
        expected = "var disqus_title = '%s';" % page.title
        self.assertIn(expected, out)

    def test_get_page_url_for_language(self):
        """
        Test the get_page_url_for_language tag
        """
        template = lambda language_code: "".join(
            [
                "{% load td_cms_tags %}",
                "{% get_page_url_for_language page ",
                "'%s'" % language_code,
                " as url %}",
                "{{ url }}"
            ]
        )

        # Test page
        page = Page.objects.get(title='Django book (vol. 10)')

        # Test the french slug
        out = Template(template('fr')).render(Context({'page': page}))
        expected = u'/'
        self.assertEqual(out, expected)

        # Define french titles
        page.title_fr = u'Le livre Django (vol. 10)'
        page.save()

        category = page.categories.all()[0]
        category.name_fr = u'Jazz avec Django'
        category.save()

        # Test the english (default) slug
        out = Template(template('en')).render(Context({'page': page}))
        expected = u'/en/jazz-with-django/django-book-vol-10/'
        self.assertEqual(out, expected)

        # Test the french slug
        out = Template(template('fr')).render(Context({'page': page}))
        expected = u'/fr/jazz-avec-django/le-livre-django-vol-10/'
        self.assertEqual(out, expected)

        # Test with a wrong language code
        with self.assertRaises(ImproperlyConfigured):
            Template(template('de')).render(Context({'page': page}))

        # Test with a None object page
        with self.assertRaises(ObjectDoesNotExist):
            Template(template('fr')).render(Context({'page': None}))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Category'
        db.create_table(u'td_cms_category', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('name_en', self.gf('django.db.models.fields.CharField')(max_length=100, unique=True, null=True, blank=True)),
            ('name_fr', self.gf('django.db.models.fields.CharField')(max_length=100, unique=True, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=100, blank=True)),
            ('short_description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('short_description_en', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('short_description_fr', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('description_en', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('description_fr', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('parent', self.gf('mptt.fields.TreeForeignKey')(blank=True, related_name='children', null=True, to=orm['td_cms.Category'])),
            ('visible_in_menu', self.gf('django.db.models.fields.BooleanField')(default=True)),
            (u'lft', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'rght', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'tree_id', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
            (u'level', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True)),
        ))
        db.send_create_signal(u'td_cms', ['Category'])

        # Adding model 'Page'
        db.create_table(u'td_cms_page', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('title_en', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('title_fr', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=200, blank=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('synopsis', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('synopsis_en', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('synopsis_fr', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('image', self.gf('filebrowser.fields.FileBrowseField')(max_length=250, null=True, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('content_en', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('content_fr', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.CharField')(default='draft', max_length=50)),
            ('is_highlighted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('last_modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'td_cms', ['Page'])

        # Adding M2M table for field categories on 'Page'
        m2m_table_name = db.shorten_name(u'td_cms_page_categories')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('page', models.ForeignKey(orm[u'td_cms.page'], null=False)),
            ('category', models.ForeignKey(orm[u'td_cms.category'], null=False))
        ))
        db.create_unique(m2m_table_name, ['page_id', 'category_id'])


    def backwards(self, orm):
        # Deleting model 'Category'
        db.delete_table(u'td_cms_category')

        # Deleting model 'Page'
        db.delete_table(u'td_cms_page')

        # Removing M2M table for field categories on 'Page'
        db.delete_table(db.shorten_name(u'td_cms_page_categories'))


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'td_cms.category': {
            'Meta': {'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'description_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'description_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'name_en': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'name_fr': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': u"orm['td_cms.Category']"}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'short_description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'short_description_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'short_description_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'blank': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'visible_in_menu': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'td_cms.page': {
            'Meta': {'object_name': 'Page'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'page'", 'symmetrical': 'False', 'to': u"orm['td_cms.Category']"}),
            'content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'content_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'content_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'is_highlighted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'auto_now_add': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'draft'", 'max_length': '50'}),
            'synopsis': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'synopsis_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'synopsis_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'title_en': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title_fr': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['td_cms']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Page.allow_sharing'
        db.add_column(u'td_cms_page', 'allow_sharing',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Page.allow_sharing'
        db.delete_column(u'td_cms_page', 'allow_sharing')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'td_cms.category': {
            'Meta': {'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'description_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'description_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            u'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            u'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'name_en': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'name_fr': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'parent': ('mptt.fields.TreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': u"orm['td_cms.Category']"}),
            'position': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            u'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'short_description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'short_description_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'short_description_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'blank': 'True'}),
            'slug_en': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'slug_fr': ('django.db.models.fields.SlugField', [], {'max_length': '100', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            u'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'visible_in_menu': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'td_cms.page': {
            'Meta': {'ordering': "['position']", 'object_name': 'Page'},
            'allow_comments': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_sharing': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'categories': ('mptt.fields.TreeManyToManyField', [], {'related_name': "'page'", 'symmetrical': 'False', 'to': u"orm['td_cms.Category']"}),
            'content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'content_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'content_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'is_highlighted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'auto_now_add': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'slug_en': ('django.db.models.fields.SlugField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'slug_fr': ('django.db.models.fields.SlugField', [], {'max_length': '200', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'draft'", 'max_length': '50'}),
            'synopsis': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'synopsis_en': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'synopsis_fr': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'title_en': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title_fr': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['td_cms']


# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.translation import ugettext as _

from mptt.admin import MPTTModelAdmin
from modeltranslation.admin import TranslationAdmin
from reversion import VersionAdmin

from .models import Category, Page


class TinyMCEAdminMixin(object):
    group_fieldsets = True

    class Media:
        js = [
            '/static/grappelli/tinymce/jscripts/tiny_mce/tiny_mce.js',
            '/static/td_cms/javascripts/vendor/tinymce_setup.js',
        ]


class CategoryAdmin(TinyMCEAdminMixin, TranslationAdmin, MPTTModelAdmin):
    fieldsets = (
        (_("Base"), {'fields': ('name', 'parent', 'position',
                                'visible_in_menu')}),
        (_("Short description"), {'fields': ('short_description',)}),
        (_("Description"), {'fields': ('description',)}),
    )


class PageAdmin(TinyMCEAdminMixin, TranslationAdmin, VersionAdmin):
    fieldsets = (
        (_("Base"), {'fields': ('title', 'image', 'categories', 'status',
                                'is_highlighted', 'allow_comments',
                                'allow_sharing', 'position', 'author')}),
        (_("Synopsis"), {'fields': ('synopsis',)}),
        (_("Content"), {'fields': ('content',)}),
    )
    list_display = ('title', 'author', 'status', 'is_highlighted',
                    'creation_date', 'last_modified')
    list_display_links = ('title',)
    list_filter = ('status', 'creation_date', 'last_modified')
    actions = ['make_published', 'make_highlighted']

    def save_form(self, request, form, change):
        obj = super(PageAdmin, self).save_form(request, form, change)
        if not change:
            obj.author = request.user
        return obj

    # Actions
    def make_published(modeladmin, request, queryset):
        queryset.update(status='published')
    make_published.short_description = _("Mark selected pages as published")

    def make_highlighted(modeladmin, request, queryset):
        queryset.update(is_highlighted=True)
    make_highlighted.short_description = _("Mark selected pages as "
                                           "highlighted")


admin.site.register(Category, CategoryAdmin)
admin.site.register(Page, PageAdmin)



# -*- coding: utf-8 -*-
import factory
import factory.fuzzy
from loremipsum import get_sentence, get_paragraphs

from . import models


def lorem_paragraphs(p):
    s = '<p>'
    s += '</p><p>\n'.join(get_paragraphs(p))
    s += '</p>'
    return s


class CategoryFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = models.Category
    FACTORY_DJANGO_GET_OR_CREATE = ('name',)

    name = factory.Sequence(lambda n: u"Category {0}".format(n))
    short_description = lorem_paragraphs(1)
    description = lorem_paragraphs(2)


class PageFactory(factory.django.DjangoModelFactory):
    FACTORY_FOR = models.Page
    FACTORY_DJANGO_GET_OR_CREATE = ('title',)

    title = unicode(get_sentence())
    synopsis = lorem_paragraphs(1)
    content = lorem_paragraphs(10)

    @factory.post_generation
    def categories(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of groups were passed in, use them
            for category in extracted:
                self.categories.add(category)



# -*- coding: utf-8 -*-
"""
Urls for testing purpose
"""
from django.conf.urls import include, patterns, url

from .views import NotCMSTestView, RootTestView


urlpatterns = patterns('',
    # Test views
    url(r'^$',
        RootTestView.as_view(),
        name='root_view'),

    url(r'^this/is/a/testing_pattern/',
        NotCMSTestView.as_view(),
        name='not_cms_test_view'),

    # TD_CMS views
    url(r'', include('td_cms.urls')),
)



# -*- coding: utf-8 -*-
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.core.urlresolvers import resolve
from django.http import Http404
from django.utils.http import is_safe_url
from django.utils.translation import (
    activate as translation_activate, check_for_language, ugettext as _
)
from django.views.generic import DetailView, RedirectView
from modeltranslation.settings import AVAILABLE_LANGUAGES
from purl import URL

from .models import Category, Page


class TranslatedSlugMixin(object):
    """
    Returns the object the view is displaying, given a translated slug.
    """

    def get_object(self, queryset=None):
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Available languages translated slug field name and value
        for language in AVAILABLE_LANGUAGES:
            slug_field = 'slug_%s' % language

            # The translated slug value matching our url pattern
            slug = self.kwargs.get(slug_field, None)

            if slug:
                break

        # Our object
        queryset = queryset.filter(**{slug_field: slug})

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except ObjectDoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


class SetLanguageView(RedirectView):
    """
    Mimic the django default set_language i18n view with language specific url
    support
    """
    permanent = False

    def get_redirect_url(self, **kwargs):
        """
        This is the core part
        """
        path = URL(self.request.META.get('HTTP_REFERER')).path()
        try:
            match = resolve(path)
        except Http404:
            return '/'

        next = self.request.REQUEST.get('next')

        # Get url slug
        fieldset = dict((k, v) for k, v in match.kwargs.iteritems() if 'slug_' in k)  # NOPEP8
        if fieldset:
            # Instantiate the view
            #
            # Known issue: this may not work with function based views
            exec('view_class = %s()' % match.func.func_name)
            if hasattr(view_class, 'model'):  # NOQA
                obj = view_class.model.objects.get(**fieldset)  # NOQA
                next = obj.get_absolute_url()

        # Check url safety
        if not is_safe_url(url=next, host=self.request.get_host()):
            next = '/'

        return next

    def post(self, request, *args, **kwargs):
        """
        Get target language and save this parameter to user session
        """
        lang_code = request.POST.get('language', None)
        if lang_code and check_for_language(lang_code):
            if not hasattr(request, 'session'):
                raise ImproperlyConfigured(
                    'td_cms requires the django.contrib.sessions app '
                    'installed and configured'
                )
            request.session['django_language'] = lang_code
            translation_activate(lang_code)
        return super(SetLanguageView, self).post(request, *args, **kwargs)


class CategoryDetailView(TranslatedSlugMixin, DetailView):
    model = Category
    template_name = 'td_cms/category_detail.html'


class PageDetailView(TranslatedSlugMixin, DetailView):
    model = Page
    template_name = 'td_cms/page_detail.html'



# -*- coding: utf-8 -*-
from modeltranslation.translator import translator, TranslationOptions
from .models import Category, Page


class CategoryTranslationOptions(TranslationOptions):
    fields = ('name', 'slug', 'short_description', 'description',)


class PageTranslationOptions(TranslationOptions):
    fields = ('title', 'slug', 'synopsis', 'content',)


translator.register(Category, CategoryTranslationOptions)
translator.register(Page, PageTranslationOptions)



# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test views
"""
from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.translation import activate, get_language

from ..models import Page
from .mixins import TDCMSTestMixin


class SetLanguageViewTests(TDCMSTestMixin, TestCase):
    """
    Tests for the SetLanguageView
    """
    def setUp(self):
        # We reset english as default
        activate('en')

        self._set_category()
        self._set_page(categories=(self.category, ))

        # Set translation
        self.category.name_fr = u"Catégorie 1"
        self.category.save()

        self.page.title_fr = u"Titre 1"
        self.page.save()

    def test_redirection_for_a_page(self):
        """
        Test the post method for a page

        We receive the language code and the next url from the set language
        form
        """
        # This is the default language
        self.assertEqual(get_language(), u'en')

        # Post data to switch the language and follow redirect
        url = reverse('td_cms_set_language')
        response = self.client.post(
            url,
            {
                'next': '/',
                'language': 'fr',
            },
            follow=True,
            HTTP_REFERER=self.page.get_absolute_url(),
        )
        self.assertEqual(response.status_code, 200)

        # This is the translated url were we should have been redirected
        expected_path = u'/fr/categorie-1/titre-1/'
        self.assertEqual(response.request['PATH_INFO'], expected_path)

        # Check that the language has been updated
        self.assertEqual(get_language(), u'fr')

        # Now switch back to en
        response = self.client.post(
            url,
            {
                'next': '/',
                'language': 'en',
            },
            follow=True,
            HTTP_REFERER=self.page.get_absolute_url(),
        )
        self.assertEqual(response.status_code, 200)

        # This is the translated url were we should have been redirected
        expected_path = u'/en/category-1/title-1/'
        self.assertEqual(response.request['PATH_INFO'], expected_path)

        # Check that the language has been updated
        self.assertEqual(get_language(), u'en')

    def test_redirection_for_a_category(self):
        """
        Test the post method for a category

        We receive the language code and the next url from the set language
        form
        """
        # This is the default language
        self.assertEqual(get_language(), u'en')

        # Post data to switch the language and follow redirect
        url = reverse('td_cms_set_language')
        response = self.client.post(
            url,
            {
                'next': '/',
                'language': 'fr',
            },
            follow=True,
            HTTP_REFERER=self.category.get_absolute_url(),
        )
        self.assertEqual(response.status_code, 200)

        # This is the translated url were we should have been redirected
        expected_path = u'/fr/categorie-1/'
        self.assertEqual(response.request['PATH_INFO'], expected_path)

        # Check that the language has been updated
        self.assertEqual(get_language(), u'fr')

        # Switch back to en
        response = self.client.post(
            url,
            {
                'next': '/',
                'language': 'en',
            },
            follow=True,
            HTTP_REFERER=self.category.get_absolute_url(),
        )
        self.assertEqual(response.status_code, 200)

        # This is the translated url were we should have been redirected
        expected_path = u'/en/category-1/'
        self.assertEqual(response.request['PATH_INFO'], expected_path)

        # Check that the language has been updated
        self.assertEqual(get_language(), u'en')

    def test_redirection_for_a_non_cms_page(self):
        """
        Test redirection for an existing url that is not a page or a category
        """
        url = reverse('td_cms_set_language')
        http_referer = reverse('not_cms_test_view')

        response = self.client.post(
            url,
            {
                'next': http_referer,
                'language': 'fr',
            },
            follow=True,
            HTTP_REFERER=http_referer,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.request['PATH_INFO'], http_referer)

    def test_redirection_from_an_unknown_path(self):
        """
        Test redirection from a 404 http referer
        """
        url = reverse('td_cms_set_language')
        http_referer = u'/nowhere/'

        response = self.client.post(
            url,
            {
                'next': http_referer,
                'language': 'fr',
            },
            follow=True,
            HTTP_REFERER=http_referer,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.request['PATH_INFO'], '/')

    def test_redirection_to_an_unsafe_url(self):
        """
        Test redirection to an 'unsafe' url
        """
        url = reverse('td_cms_set_language')
        http_referer = reverse('not_cms_test_view')

        response = self.client.post(
            url,
            {
                'next': 'http://comsource.fr',
                'language': 'fr',
            },
            follow=True,
            HTTP_REFERER=http_referer,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.request['PATH_INFO'], '/')

    @override_settings(
        INSTALLED_APPS=(),
        MIDDLEWARE_CLASSES=(),
    )
    def test_redirection_without_active_session(self):
        """
        Test redirection without an active session
        """
        # This is the default language
        self.assertEqual(get_language(), u'en')

        # Post data to switch the language and follow redirect
        url = reverse('td_cms_set_language')
        with self.assertRaises(ImproperlyConfigured):
            self.client.post(
                url,
                {
                    'next': '/',
                    'language': 'fr',
                },
                follow=True,
                HTTP_REFERER=self.page.get_absolute_url(),
            )


class CategoryDetailViewTests(TDCMSTestMixin, TestCase):
    """
    Tests for the CategoryDetailView
    """
    def setUp(self):
        self._set_category()
        self._add_pages()

    def test_get(self):
        """
        Test the CategoryDetailView get method
        """
        response = self.client.get(self.category.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['category'], self.category)

    def test_get_unknown_slug(self):
        """
        Test the CategoryDetailView get method with an unknown object
        """
        kwargs = {
            'slug_en': 'this-is-a-wrong-slug',
        }
        url = reverse('category_detail_en', kwargs=kwargs)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_css_styles_are_included(self):
        """
        Test TailorDev CMS css styles are loaded
        """
        response = self.client.get(self.category.get_absolute_url())

        css_link = ''.join(
            '<link rel="stylesheet" href="/static/td_cms/css/styles.css" />'
        )
        self.assertContains(response, css_link)

    def test_get_with_a_different_language_from_session(self):
        """
        Test a category detailled view display for another language
        """
        self.category.name_fr = u"Catégorie 1"
        self.category.save()

        url_fr = "/fr/categorie-1/"
        url_en = "/en/category-1/"

        response = self.client.get(url_en)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['category'], self.category)

        response = self.client.get(url_fr)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['category'], self.category)


class PageDetailViewTests(TDCMSTestMixin, TestCase):
    """
    Tests for the CategoryDetailView
    """
    def setUp(self):
        self._set_category()
        self._add_pages()

    def test_get(self):
        """
        Test the PageDetailView get method
        """
        page = Page.objects.get(id=1)
        response = self.client.get(page.get_absolute_url())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['page'], page)

    def test_get_unknown_slug(self):
        """
        Test the PageDetailView get method with an unknown object
        """
        kwargs = {
            'category_slug': self.category.slug,
            'slug_en': 'this-is-a-wrong-slug',
        }
        url = reverse('page_detail_en', kwargs=kwargs)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_css_styles_are_included(self):
        """
        Test TailorDev CMS css styles are loaded
        """
        page = Page.objects.get(id=1)
        response = self.client.get(page.get_absolute_url())

        css_link = ''.join(
            '<link rel="stylesheet" href="/static/td_cms/css/styles.css" />'
        )
        self.assertContains(response, css_link)

    def test_get_with_a_different_language_from_session(self):
        """
        Test a page detailled view display for another language
        """
        self.category.name_fr = u"Catégorie 1"
        self.category.save()

        page = self.category.page.get(slug=u'django-book-vol-1')
        page.title_fr = u"Le livre Django (vol. 1)"
        page.save()

        url_fr = "/fr/categorie-1/le-livre-django-vol-1/"
        url_en = "/en/category-1/django-book-vol-1/"

        response = self.client.get(url_en)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['page'], page)

        response = self.client.get(url_fr)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['page'], page)



# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test commands.
"""
from django.test import TestCase

from ..management.commands import cms_lorem_fixtures
from ..models import Category, Page


class CMSLoremFixturesCommandTests(TestCase):
    """
    Tests for the cms_lorem_fixtures command
    """
    def test_command(self):
        """
        Test that the command creates pages and categories
        """
        cmd = cms_lorem_fixtures.Command()
        cmd.handle()

        self.assertEqual(Category.objects.count(), 9)
        self.assertEqual(Page.objects.count(), 18)



# -*- coding: utf-8 -*-
from django.conf.urls import url, patterns

from modeltranslation.settings import AVAILABLE_LANGUAGES

from .views import CategoryDetailView, PageDetailView, SetLanguageView


urlpatterns = patterns('',

    # Set language view
    url(r'^i18n/td_cms_set_language/',
        SetLanguageView.as_view(),
        name='td_cms_set_language')
)


# Define all urls given available languages
for language_code in AVAILABLE_LANGUAGES:
    # All urls are prefixed by the language code for translated slugs
    # resolution
    base_url = r'^%s/' % language_code

    urlpatterns += patterns('',

        # Category
        url(base_url + r'(?P<slug_%s>[-\w]+)/$' % language_code,
            CategoryDetailView.as_view(),
            name='category_detail_%s' % language_code),

        # Page
        url(
            base_url + r'(?P<category_slug>[-\w]+)/(?P<slug_%s>[-\w]+)/$' % (
                language_code
            ),
            PageDetailView.as_view(),
            name='page_detail_%s' % language_code),
    )



# -*- coding: utf-8 -*-
import logging

from datetime import datetime
from django import template
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from modeltranslation.settings import AVAILABLE_LANGUAGES

from td_cms.models import Category, Page

register = template.Library()
logger = logging.getLogger(__name__)


@register.assignment_tag
def get_parent_categories():
    """
    Add TD CMS page categories to the context
    """
    return Category.visible_objects.all()


@register.assignment_tag
def get_highlighted_pages():
    """
    Add TD CMS highlighted pages to the context
    """
    return Page.highlighted_objects.all()


@register.assignment_tag
def get_category(slug):
    """
    Add a category to the context given its slug
    """
    for language_code in AVAILABLE_LANGUAGES:
        slug_field = 'slug_%s' % language_code
        try:
            return Category.objects.get(**{slug_field: slug})
        except:
            pass
    logger.warning("Cannot resolve category slug '%s'" % slug)
    return None


@register.assignment_tag
def get_page(slug):
    """
    Add a page to the context given its slug
    """
    for language_code in AVAILABLE_LANGUAGES:
        slug_field = 'slug_%s' % language_code
        try:
            return Page.objects.get(**{slug_field: slug})
        except:
            pass
    logger.warning("Cannot resolve page slug '%s'" % slug)
    return None


@register.assignment_tag
def get_page_url_for_language(page, language_code):
    """
    Get page url for a speficied language code.

    Returns: the url or '/' if a NoReverseMatch exception is raised

    ex: get_page_url_for_language page fr as url
    """
    if language_code not in AVAILABLE_LANGUAGES:
        msg = "Selected language %s is not available" % language_code
        raise ImproperlyConfigured(msg)

    if page is None:
        msg = "Input page does not exists"
        raise ObjectDoesNotExist(msg)

    category = page.categories.all()[0]
    view_name = 'page_detail_%s' % language_code
    category_slug = getattr(category, 'slug_%s' % language_code, None)
    page_slug = getattr(page, 'slug_%s' % language_code, None)

    # Silently fail when page has not been translated
    if None in (page_slug, category_slug):
        return '/'

    return reverse(view_name, args=[category_slug, page_slug])


@register.filter
def timestamp(value):
    """
    Convert a datetime object representation to a unix timestamp.

    If the object to filter is not a valid datetime.datetime instance, returns
    the original unmodified object.
    """
    if not isinstance(value, datetime):
        return value
    return value.strftime("%s")


@register.inclusion_tag(
    'td_cms/partials/comments/disqus_thread.html',
    takes_context=True
)
def show_disqus_thread(context):
    """
    Show the disqus thread for a Page that allows comments
    """
    if not getattr(settings, 'TD_CMS_DISQUS_SHORTNAME', None):
        msg = _('TD_CMS_DISQUS_SHORTNAME parameter is missing in your '
                'settings')
        messages.warning(context['request'], msg)
        return context

    context.update(
        {
            'disqus_shortname': settings.TD_CMS_DISQUS_SHORTNAME,
        }
    )
    return context



# -*- coding: utf-8 -*-
from django.views.generic import TemplateView


class NotCMSTestView(TemplateView):
    """
    A simple view used for testing purpose
    """
    template_name = '_layouts/base.html'


class RootTestView(TemplateView):
    """
    A root view for testing purpose
    """
    template_name = '_layouts/base.html'



# -*- coding: utf-8 -*-
"""
Django TailorDev CMS

Test models.
"""
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.test import TestCase
from django.utils.text import slugify
from django.utils.translation import activate

from . import Foo
from ..factories import CategoryFactory, PageFactory
from ..models import Category, Page
from .mixins import TDCMSTestMixin


class CategoryModelTests(TDCMSTestMixin, TestCase):
    """
    Tests for the Category model
    """
    def setUp(self):
        self._set_category()

    def test_unicode(self):
        """
        Test __unicode__ method
        """
        self.assertEqual(unicode(self.category), self.category.name)

    def test_saving_and_retrieving_items(self):
        """
        Test saving and retrieving two categories with different names
        """
        name_1 = self.category.name

        name_2 = u"Category 2"
        category_2 = Category()
        category_2.name = name_2
        category_2.save()

        categories = Category.objects.all()
        self.assertEqual(categories.count(), 2)

        saved_category_1 = categories[0]
        saved_category_2 = categories[1]
        self.assertEqual(saved_category_1.name, name_1)
        self.assertEqual(saved_category_2.name, name_2)

    def test_slug_unicity(self):
        """
        Test saving two categories with the same name
        """
        category_2 = Category()
        category_2.name = self.category.name
        self.assertRaisesMessage(
            IntegrityError,
            'column slug_en is not unique',
            category_2.save
        )

    def test_multilingual_slug(self):
        """
        Test multilingual slugification
        """
        name_en = u"Django books"
        name_fr = u"Livres de Django"

        category = Category()
        category.name_en = name_en
        category.name_fr = name_fr

        category.slugify('name')
        self.assertEqual(category.slug_en, slugify(name_en))
        self.assertEqual(category.slug_fr, slugify(name_fr))

    def test_get_absolute_url(self):
        """
        Test the get_absolute_url method (with multilingual support)
        """
        url = reverse('category_detail_en', args=[self.category.slug])
        self.assertEqual(self.category.get_absolute_url(), url)

        # Set a new language
        activate('fr')
        url = reverse('category_detail_fr', args=[self.category.slug])
        self.assertEqual(self.category.get_absolute_url(), url)

    def test_category_mptt_order(self):
        """
        Test Category tree ordering
        """
        self.category.position = 1
        self.category.save()

        name_1 = self.category.name

        name_2 = u"Must be the first"
        category_2 = Category()
        category_2.name = name_2
        category_2.position = 0
        category_2.save()

        categories = Category.objects.all()

        first_position_category = categories[0]
        second_position_category = categories[1]
        self.assertEqual(first_position_category.name, name_2)
        self.assertEqual(second_position_category.name, name_1)

    def test_get_pages(self):
        """
        Test the get_pages method
        """
        self._add_pages()

        pages = self.category.get_pages()
        self.assertEqual(len(pages), 3)

    def test_get_pages_with_heterogenous_related_models(self):
        """
        Test a category linked with a model that is not a page
        """
        category_1 = CategoryFactory()
        category_2 = CategoryFactory()

        PageFactory.create(categories=(category_1,))
        foo = Foo.objects.create(title="Bar")
        foo.categories.add(category_2)
        foo.save()

        misc = category_2.get_pages(verbose=True)
        self.assertEqual(len(misc), 1)

    def test_get_pages_with_subcategories(self):
        """
        Test the get_pages method when the main category has no linked pages
        but one of its sub-categories has linked pages.
        """
        # Create two sub-categories with 6 pages (3 highlighted pages)
        for num in range(1, 3):
            subcategory = Category.objects.create(
                name=u"Category 1.%(num)d" % {'num': num},
                parent=self.category
            )
            self._add_pages(subcategory)

        pages = self.category.get_pages()
        self.assertEqual(len(pages), 6)

    def test_get_highlighted_pages(self):
        """
        Test the get_highlighted_pages method
        """
        self._add_pages()
        page = self.category.page.all()[0]
        page.is_highlighted = True
        page.status = 'published'
        page.save()

        pages = self.category.get_highlighted_pages()
        self.assertEqual(len(pages), 1)

    def test_visible_category_manager(self):
        """
        Test the visible category manager
        """
        self._set_category_tree()
        categories = Category.objects.all()
        visible_categories = Category.visible_objects.all()

        # By default a category is visible
        self.assertEqual(categories.count(), visible_categories.count())

        # Toggle all categories visibility status
        Category.objects.all().update(visible_in_menu=False)
        visible_categories = Category.visible_objects.all()
        self.assertEqual(visible_categories.count(), 0)


class PageModelTests(TDCMSTestMixin, TestCase):
    """
    Tests for the Category model
    """
    def setUp(self):
        self._set_category()
        self._set_page(categories=[self.category])

    def test_unicode(self):
        """
        Test __unicode__ method
        """
        self.assertEqual(unicode(self.page), self.page.title)

    def test_page_factory(self):
        """
        Test the PageFactory with no linked categories
        """
        page = PageFactory.build(title=u"This is a title")
        self.assertIsNot(page, None)

    def test_saving_and_retrieving_items(self):
        """
        Test saving and retrieving two categories with different names
        """
        title_1 = self.page.title

        title_2 = u"Title 2"
        page_2 = Page()
        page_2.title = title_2
        page_2.save()

        pages = Page.objects.all()
        self.assertEqual(pages.count(), 2)

        saved_page_1 = pages[0]
        saved_page_2 = pages[1]
        self.assertEqual(saved_page_1.title, title_1)
        self.assertEqual(saved_page_2.title, title_2)

    def test_get_absolute_url(self):
        """
        Test the get_absolute_url method (with multilingual support)
        """
        page_slug = self.page.slug
        category_slug = self.page.categories.all()[0].slug

        url = reverse('page_detail_en', args=[category_slug, page_slug])
        self.assertEqual(self.page.get_absolute_url(), url)

        # Set a new language
        activate('fr')
        url = reverse('page_detail_fr', args=[category_slug, page_slug])
        self.assertEqual(self.page.get_absolute_url(), url)

    def test_get_current_site(self):
        """
        Test the get_current_site method from the SiteMixin
        """
        site = self.page.get_current_site()
        self.assertEqual(site.domain, 'example.com')
        self.assertEqual(site.name, 'example.com')

    def test_get_image_full_url(self):
        """
        Test the get_image_full_url method
        """
        # No image has been defined
        image_full_url = self.page.get_image_full_url()
        self.assertEqual(image_full_url, '')

        # Define an image
        image_relative_url = '/media/upload/foo/test.jpg'
        self.page.image = image_relative_url
        self.page.save()
        image_full_url = self.page.get_image_full_url()

        site = self.page.get_current_site()
        url = "http://%(domain)s%(image)s" % {
            'domain': site.domain,
            'image': image_relative_url,
        }
        self.assertEqual(url, image_full_url)

    def test_published_page_manager(self):
        """
        Test the published page manager
        """
        self._set_category_tree()
        published_pages = Page.published_objects.all()

        self.assertEqual(published_pages.count(), 18)

    def test_highlighted_page_manager(self):
        """
        Test the highlighted page manager
        """
        self._set_category_tree()
        highlighted_pages = Page.highlighted_objects.all()

        self.assertEqual(highlighted_pages.count(), 4)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import get_language, ugettext as _
from django.utils.text import slugify

from filebrowser.fields import FileBrowseField
from modeltranslation.settings import AVAILABLE_LANGUAGES, DEFAULT_LANGUAGE
from mptt.fields import TreeForeignKey, TreeManyToManyField
from mptt.models import MPTTModel
from mptt.managers import TreeManager


# Custom Managers
#
class VisibleCategoryManager(models.Manager):
    def get_query_set(self):
        return super(VisibleCategoryManager, self).get_query_set().filter(
            visible_in_menu=True)


class PublishedPageManager(models.Manager):
    def get_query_set(self):
        return super(PublishedPageManager, self).get_query_set().filter(
            status='published')


class HighlightedPageManager(models.Manager):
    def get_query_set(self):
        return super(HighlightedPageManager, self).get_query_set().filter(
            status='published', is_highlighted=True)


# Mixins
#
class SiteMixin(object):
    """
    Add site-framework related features to a model
    """

    def get_current_site(self):
        return Site.objects.get_current()


class TranslatableSlugMixin(object):

    def slugify(self, field, slug_field='slug'):
        """
        Generate slugs given:
        * all available languages
        * a given slugifyable field
        """

        # Generate all languages slugs
        for language_code in AVAILABLE_LANGUAGES:
            src_field = '%s_%s' % (field, language_code)
            aslug_field = '%s_%s' % (slug_field, language_code)

            src_value = getattr(self, src_field, None)

            if not src_value:
                continue

            # Current language slug
            setattr(self, aslug_field, slugify(src_value))

            if not language_code == DEFAULT_LANGUAGE:
                continue

            # Set default language
            setattr(self, slug_field, slugify(src_value))


# Abstract Models
#
class AbstractCategory(TranslatableSlugMixin, MPTTModel):
    """
    Page category model (Hierarchical tree)
    """
    name = models.CharField(_("Name"), max_length=100, unique=True)
    slug = models.SlugField(_("Slug"), max_length=100,
                            help_text=_("You are not supposed to fill this"),
                            blank=True, unique=True)
    short_description = models.TextField(_("Short description"),
                                         blank=True, null=True)
    description = models.TextField(_("Description"),
                                   help_text=_("Describe this category here"),
                                   blank=True, null=True)
    parent = TreeForeignKey('self', null=True, blank=True,
                            related_name='children')
    position = models.IntegerField(_("Position"),
                                   help_text=_("Used for objects ordering"),
                                   default=0)
    visible_in_menu = models.BooleanField(_("Visible in main menu?"),
                                          default=True)
    # Managers
    objects = TreeManager()
    visible_objects = VisibleCategoryManager()

    class Meta:
        abstract = True
        verbose_name = _("Category")
        verbose_name_plural = _("Categories")

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        """
        Generate slugs for all languages before saving
        """
        self.slugify('name')
        super(AbstractCategory, self).save(*args, **kwargs)

    @models.permalink
    def get_absolute_url(self):
        language = get_language().split('-')[0]
        return ('category_detail_%s' % language, [self.slug])

    class MPTTMeta:
        order_insertion_by = ['position']

    def get_pages(self, verbose=False):
        # This is an abstract models, hence, various models may
        # be related to the Category model.
        #
        # Return a list, not a queryset
        qss = list()
        for related in self._meta.get_all_related_many_to_many_objects():
            manager = getattr(self, related.get_accessor_name(), None)
            if not manager:
                continue
            qs = manager.all()
            if not qs and not self.is_leaf_node():
                children = self.get_children()
                qs = related.model.objects.filter(categories__in=children)
            # Filter only pages related to categories
            if related.name == u'td_cms:page':
                qs = qs.filter(status='published').order_by('position')
            if len(qs):
                qss += qs
        return qss

    def get_highlighted_pages(self):
        qss = self.get_pages()
        return [p for p in qss if p.is_highlighted]


PAGE_STATUS = (
    ('draft', _("Draft")),
    ('private', _("Private")),
    ('published', _("Published")),
)


class AbstractPage(TranslatableSlugMixin, SiteMixin, models.Model):
    """
    Page model
    """
    title = models.CharField(_("Title"), max_length=200)
    slug = models.SlugField(_("Slug"), max_length=200,
                            help_text=_("You are not supposed to fill this"),
                            blank=True, unique=True)
    author = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    synopsis = models.TextField(_("Page synopsis"), blank=True, null=True)
    image = FileBrowseField(_("Page main image"), max_length=250,
                            directory='images/pages/',
                            extensions=['.jpg', '.png', '.gif', '.svg'],
                            blank=True, null=True)
    content = models.TextField(_("Page content"),
                               help_text=_("Describe this category here"),
                               blank=True, null=True)

    categories = TreeManyToManyField('td_cms.Category',
                                     related_name="%(class)s")
    status = models.CharField(_("Status"), choices=PAGE_STATUS,
                              default=PAGE_STATUS[0][0], max_length=50)
    is_highlighted = models.BooleanField(_("Highlighted article?"),
                                         help_text=_("If selected, this "
                                                     "article may be shown on "
                                                     "sidebars or the home "
                                                     "page"),
                                         default=False)
    allow_comments = models.BooleanField(_("Allow comments"), default=False)
    allow_sharing = models.BooleanField(_("Allow social media sharing"),
                                        default=False)
    position = models.IntegerField(_("Position"), default=0)
    creation_date = models.DateTimeField(_("Creation date"), auto_now_add=True)
    last_modified = models.DateTimeField(_("Last modified"),
                                         auto_now=True, auto_now_add=True)
    # Managers
    objects = models.Manager()
    published_objects = PublishedPageManager()
    highlighted_objects = HighlightedPageManager()

    class Meta:
        abstract = True
        ordering = ['position', ]
        verbose_name = _("Page")
        verbose_name_plural = _("Pages")

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        """
        Generate slugs for all languages before saving
        """
        self.slugify('title')
        super(AbstractPage, self).save(*args, **kwargs)

    @models.permalink
    def get_absolute_url(self):
        language = get_language().split('-')[0]
        category = self.categories.all()[0]
        return ('page_detail_%s' % language, [category.slug, self.slug])

    def get_image_full_url(self):
        """
        Return a fully qualified url with domain. If no image has been defined,
        returns None.
        """
        if not self.image:
            return ""

        site = self.get_current_site()
        return "http://%s%s" % (site.domain, self.image.url)



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
import logging
from loremipsum import get_sentence

from td_cms.factories import CategoryFactory, PageFactory


LOREM_CATEGORIES = (
    # ('Main Category', ('Related', 'Sub categories'))
    (u'Company', (u'Research & Development', u'Marketing')),
    (u'Activity', (u'Genomics', u'Bioinformatics')),
    (u'News', (u'Scientific News', u'Business News')),
)


class Command(BaseCommand):
    args = ''
    help = 'Generates lorem ipsum fixtures for designers'

    def page(self, logger):
        """
        Page app fixtures
        """

        # Parent categories
        for category in LOREM_CATEGORIES:
            parent_category = CategoryFactory.create(name=category[0])
            logger.info('New parent categories: %s', parent_category)

            # Child categories
            for child_category_name in category[1]:
                child_category = CategoryFactory.create(
                    name=child_category_name,
                    parent=parent_category)
                logger.info('New child categories: %s', child_category)

                for p in range(3):
                    PageFactory(title=unicode(get_sentence()),
                                categories=(child_category,),
                                status='published')
                logger.info('Created %s pages', child_category)

    def handle(self, *args, **options):
        logger = self._get_logger()

        # App fixtures
        self.page(logger)

    def _get_logger(self):
        logger = logging.getLogger('td_cms')
        return logger



# -*- coding: utf-8 -*-
import os

from pip.req import parse_requirements
from setuptools import setup, find_packages

REQUIREMENTS = 'requirements.txt'


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


def get_dependencies(requirements):
    """Return project dependencies as read from the requirements file"""
    r = parse_requirements(requirements)
    return [str(d.req) for d in r]


setup(
    name='django-tailordev-cms',
    version=__import__('td_cms').__version__,
    author='Julien Maupetit',
    author_email='julien@tailordev.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://bitbucket.org/tailordev/django-tailordev-cms',
    license='MIT',
    description=u' '.join(__import__('td_cms').__doc__.splitlines()).strip(),
    long_description=read_file('README.md'),
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    install_requires=get_dependencies(REQUIREMENTS),
    test_suite="runtests.runtests",
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.utils.translation import activate

from ..factories import CategoryFactory, PageFactory
from ..models import Page

TEST_CATEGORIES = (
    (u'Django books', (u'Testing in Django', u'Jazz with Django')),
    (u'Django tee', (u'Keep calm', u'Poney ride')),
    (u'Django stickers', (u'Poney mascotte', u'Django logo')),
)


class TDCMSTestMixin(object):
    """
    A mixin for category model and views testing.
    """
    def _set_user(self):
        """
        Create a default author for pages
        """
        User = get_user_model()
        self.user = User.objects.create(
            username="johndoe",
            first_name="John",
            last_name="Doe",
        )

    def _set_category(self):
        """
        Create a category and make it an attribute
        """
        # Force the default language
        activate('en')
        self.category = CategoryFactory.create(name=u"Category 1")

    def _set_page(self, categories=None):
        """
        Create a page and make it an attribute. This page is linked to the
        categories list argument.
        """
        # Force the default language
        activate('en')
        self.page = PageFactory.create(title=u"Title 1", categories=categories)

    def _add_pages(self, category=None):
        """
        Add pages to the default category
        """
        if category is None:
            category = self.category

        if not hasattr(self, 'user'):
            self._set_user()

        # Counter start
        n = Page.objects.count()

        # Create pages for the default category
        for num in xrange(n + 1, n + 7):
            title = u"Django book (vol. %(num)d)" % {'num': num}
            status = 'draft'
            highlighted = False
            if not num % 2:
                status = 'published'
            if not num % 8:
                highlighted = True

            PageFactory(title=title,
                        categories=(category,),
                        status=status,
                        is_highlighted=highlighted,
                        author=self.user)

    def _set_category_tree(self):
        """
        Set a typical category tree
        """
        for parent_category, subcategories in TEST_CATEGORIES:
            parent = CategoryFactory.create(name=parent_category)

            for subcategory in subcategories:
                sc = CategoryFactory.create(name=subcategory, parent=parent)
                self._add_pages(category=sc)



# -*- coding: utf-8 -*-
"""
Models loaded for testing purpose
"""
from django.db import models
from mptt.fields import TreeManyToManyField

from ..models import Category


class Foo(models.Model):
    """
    Foo model is only defined here for testing purpose
    """
    title = models.CharField(max_length=200)
    categories = TreeManyToManyField(Category, related_name='foos',
                                     null=True, blank=True)
